
%{
#include "rumd/Sample.h"
#include "rumd/Potential.h"
%}


// the following is to allow proper handling of the default argument "w".
// The string typemap does not seem to be applied when there are default
// arguments and I couldn't figure out how to change that

%feature("shadow") Sample::WriteConf(const std::string&filename, 
				     const std::string& mode) %{
  def WriteConf(self, filename, mode="w"):
      _rumd.Sample_WriteConf(self, filename, mode)
    %}

class Sample
{
 public:
  Sample( unsigned int set_particles_per_block, unsigned int set_threads_per_particle, bool allow_tp_280_large=false);
  Sample* Copy();
  int GetNumberOfParticles() const;
  unsigned int GetNumberOfTypes() const;
  unsigned int GetNumberThisType(unsigned int type) const;
  SortingScheme GetSortingScheme() const;

  void SetVerbose(bool vb);
  void InitializeOutputManagers(unsigned long int timeStepIndex);
  void NotifyOutputManagers(unsigned long int timeStepIndex);
  void TerminateOutputManagers();
  void ReadConf( const std::string& filename );
  void ReadRestartConf(int restartBlock, unsigned int numDigitsBlkIdx=4);
  void WriteConf(const std::string&filename, const std::string& mode);

  void SetOutputBlockSize(unsigned long int blockSize);
  void SetWriteEnergies(bool writeEnergies);
  void SetWriteTrajectory(bool writeTrajectory);
  void SetTrajectoryLinearWriteInterval(unsigned long int writeInterval);
  void SetTrajectoryLogLinParams(unsigned int base, unsigned int maxInterval);
  void SetEnergyLinearWriteInterval(unsigned int writeInterval);
  void SetEnergyLogLinParams(unsigned int base, unsigned int maxInterval);

  void SetConfIO_Format(unsigned int ioformat); // temporary
  void SetMoleculeData(MoleculeData* md);
  MoleculeData* GetMoleculeData();
  void SortParticles();
  
  void SetPotential(Potential* potential);
  void AddPotential(Potential* potential);
  void IsotropicScaleSystem( float Rscal );
  void AnisotropicScaleSystem( float Rscal, int dir );
  void SetSimulationBox(SimulationBox* simBox);
  SimulationBox* GetSimulationBox() const;
  void SetIntegrator(Integrator* newItg){ itg = newItg; }

  void SetPB_TP( unsigned int pb, unsigned int tp );
  void SetSortingScheme( SortingScheme ss);
  unsigned int GetParticlesPerBlock();
  unsigned int GetThreadsPerParticle();
  unsigned int GetNumberOfDOFs() const;

  float CalcF(bool calc_stresses = false, bool report_time = false);
  double GetPotentialEnergy() const;
  double GetVirial() const;
  ConfigurationMetaData& GetTrajectoryConfMetaData();
  EnergiesOutputManager& GetEnergiesOutputManager();
  void SetOutputDirectory(const std::string&filename);
  void SetNumberOfDOFs( unsigned i );
  void AddOutputManager(LogLinOutputManager* om);
  const Integrator* GetIntegrator() const;
};

%extend Sample {
  void Assign(const Sample& S) {
    (*$self) = S;
  }


  PyObject* GetPositions() {
    $self->CopySimulationDataFromDevice();
    const ParticleData& pData = *$self->GetParticleData();
    int len;
    npy_intp dims[2];
    unsigned int nParticles = $self->GetNumberOfParticles();
    dims[0] = nParticles;
    dims[1] = 3;
    PyObject *posArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
    char *ptr = ((PyArrayObject*) posArray)->data;
    len = 3*sizeof(float)/sizeof(char);

    for(int pdx = 0; pdx < nParticles; pdx++)
      {
	memcpy((void*) ptr, (void*)(&(pData.h_r[pdx])), len);
	ptr += len;
      }
    
    return posArray;
  }
  PyObject* GetVelocities()  {
    $self->CopySimulationDataFromDevice();
    const ParticleData& pData = *$self->GetParticleData();
    int len;
    npy_intp dims[2];
    unsigned int nParticles = $self->GetNumberOfParticles();
    dims[0] = nParticles;
    dims[1] = 3;
    PyObject *posArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
    char *ptr = ((PyArrayObject*) posArray)->data;
    len = 3*sizeof(float)/sizeof(char);

    for(int pdx = 0; pdx < nParticles; pdx++)
      {
	memcpy((void*) ptr, (void*)(&(pData.h_v[pdx])), len);
	ptr += len;
      }
    
    return posArray;
  }
  PyObject* GetForces()  {
    $self->CopySimulationDataFromDevice();
    const ParticleData& pData = *$self->GetParticleData();
    int len;
    npy_intp dims[2];
    unsigned int nParticles = $self->GetNumberOfParticles();
    dims[0] = nParticles;
    dims[1] = 3;
    PyObject *posArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
    char *ptr = ((PyArrayObject*) posArray)->data;
    len = 3*sizeof(float)/sizeof(char);

    for(int pdx = 0; pdx < nParticles; pdx++)
      {
	memcpy((void*) ptr, (void*)(&(pData.h_f[pdx])), len);
	ptr += len;
      }
    
    return posArray;
  }
  PyObject* GetImages()  {
    $self->CopySimulationDataFromDevice();
    const ParticleData& pData = *$self->GetParticleData();
    int len;
    npy_intp dims[2];
    unsigned int nParticles = $self->GetNumberOfParticles();
    dims[0] = nParticles;
    dims[1] = 3;
    PyObject *posArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
    char *ptr = ((PyArrayObject*) posArray)->data;
    len = 3*sizeof(float)/sizeof(char);

    for(int pdx = 0; pdx < nParticles; pdx++)
      {
	memcpy((void*) ptr, (void*)(&(pData.h_im[pdx])), len);
	ptr += len;
      }
    
    return posArray;
  }
};

/*
%pythoncode %{
  SORT_X = Sample.SORT_X 
  SORT_XY = Sample.SORT_XY 
  SORT_XYZ = Sample.SORT_XYZ 
    %}
*/
