import sys
sys.path.insert(0,"../../../Swig")
sys.path.insert(0,"../../../Python")
sys.path.insert(0,"../../../Tools")

import rumdSimulation, rumd, rumd_tools
# it is essential that there is no doubt about which version is actually run!
print "File used to import the rumd module",rumd.__file__
print "File used to import the rumdSimulation module",rumdSimulation.__file__
print "File used to import the rumd_tools module",rumd_tools.__file__

from rumdSimulation import rumdSimulation
from RUMD_Test import RUMD_Test

class Test_Sorting (RUMD_Test):
    """
    SPC/E with rigid bonds.
    """
    def __init__(self):
        RUMD_Test.__init__(self)
        self.numParticlesList = [1000, 8000]
        # higher tolerances on virial and related quantities due to constraints
        # not being exactly solved
        self.energy_tol["W"] = 0.0005
        self.energy_tol["p"] = 0.0005
        self.energy_tol["sxx"] = 0.0005
        self.energy_tol["syy"] = 0.0005
        self.energy_tol["szz"] = 0.0005
        
    def RunSimulation(self, mode, numParticles, pb, tp):

        # set the number of time-steps depending on the mode (assigned
        # as a class attribute so the calling function can access it later)
        self.numTimeSteps = {"short":          100,
                             "performance": 100000,
                             "stress":   100000000}[mode]
        
        # sample and simulation
        startConfg = "start.xyz.gz" 

        
        sim = rumdSimulation(startConfg, pb, tp, verbose=False)
	sim.SetSortInterval(1)
        
        if self.generate_reference_data:
            sim.SetOutputDirectory(self.ref_directory)
        else:
            sim.SetOutputDirectory(self.out_directory)

        # Create LJ pair-potential
        cut = 6.28

        # create potential object. 
        pot = rumd.Pot_LJ_12_6(cutoff_method=rumd.ShiftedForce)
        pot.SetParams(i=0, j=0, Sigma=1.0, Epsilon=1.0, Rcut=cut)
        pot.SetParams(i=1, j=0, Sigma=1.0, Epsilon=1.0, Rcut=cut)
        pot.SetParams(i=0, j=1, Sigma=1.0, Epsilon=1.0, Rcut=cut)
        pot.SetParams(i=1, j=1, Sigma=1.0, Epsilon=1.0, Rcut=cut)
        sim.SetPotential(pot)

        # remove some pair interactions
        pot.SetExclusionType(1, 1) # remove H-H inaction.
        pot.SetExclusionType(0, 1) # remove 0-H/H-0 interaction.

        pot1 = rumd.Pot_IPL_n(cutoff_method=rumd.ShiftedForce, n=1)
        pot1.SetParams(i=0, j=0, Sigma=1.0, Epsilon= 22.0199905878207*22.0199905878207, Rcut=cut)
        pot1.SetParams(i=1, j=0, Sigma=1.0, Epsilon=-22.0199905878207*11.0099952939104, Rcut=cut)
        pot1.SetParams(i=0, j=1, Sigma=1.0, Epsilon=-22.0199905878207*11.0099952939104, Rcut=cut)
        pot1.SetParams(i=1, j=1, Sigma=1.0, Epsilon= 11.0099952939104*11.0099952939104, Rcut=cut)
        sim.AddPotential(pot1)

        # integrator
        itg = rumd.IntegratorNVT(timeStep=0.0025, targetTemperature=3.80)
        sim.SetIntegrator(itg);

        # Molecule stuff
        sim.ReadMoleculeData("start.top")
        sim.SetBondConstraint(type=0, lbond=0.315855969677827)
        sim.SetBondConstraint(type=1, lbond=0.515786753581881)
        sim.moleculeData.SetNumberOfConstraintIterations(3)
                            
        # tune output depending on the mode
        if mode == "short":
            sim.SetOutputScheduling("energies","linear",interval=1)
            sim.SetOutputMetaData("energies",stress_xx=True,stress_yy=True,stress_zz=True)
            sim.SetOutputScheduling("trajectory","linear",interval=10)
            sim.SetBlockSize(50)
        elif mode == "stress":
            sim.SetOutputScheduling("energies","linear",interval=5000)
        else:
            assert mode == "performance"
            print "Note there is no output directory for performance test--the only output is the timing file"
        
        sim.Run(self.numTimeSteps,
                suppressAllOutput=(mode == "performance"))

        
if __name__ == "__main__":

    rumd_test = Test_Sorting()
    
    # standard short test to be called from makefile
    # For generating reference data and running stress and performance modes
    # there are corresponding scripts to be submitted to pbs

    sys.exit(rumd_test.RunShort("gen_ref" in sys.argv ) )
    
    # for individual runs, could do for example
    #sys.exit(rumd_test.Test("short", 1000, 16, 8))
