#ifndef LOGLINOUTPUTMANAGER_H
#define LOGLINOUTPUTMANAGER_H


#include "rumd/LogLin.h"

#include <string>
#include <sstream>
#include <iomanip>

class Sample;

class LogLinOutputManager
{
public:

  LogLinOutputManager(Sample *S, const std::string &outputDirectory, const std::string& baseFilename);
  
  virtual ~LogLinOutputManager() {}

  void SetVerbose(bool vb) { verbose = vb; }
  void SetBlockSize(unsigned long int blockSize) {this->blockSize = blockSize;}
  void SetOutputDirectory(const std::string& outputDirectory) {this->outputDirectory = outputDirectory;}
  virtual void SetLogLinParams(unsigned long int base, unsigned long int maxInterval);
  // to limit the number of items written per block 
  //virtual void SetMaxIndex(unsigned int maxIndex) {logLin.maxIndex = maxIndex;}

  virtual void Initialize(unsigned long int step);

  void SetCreatesDirectory( bool createsDirectory ) {this->createsDirectory = createsDirectory;}
  bool GetCreatesDirectory() const { return createsDirectory; }
  virtual void SetDuplicateBlockEnds(bool dbe) {duplicateBlockEnds = dbe;}
 
  virtual void Update(unsigned long int timeStep);

  virtual void Terminate() {}

private:
  LogLinOutputManager(const LogLinOutputManager&); // no default copyconstructor
  LogLinOutputManager& operator=(const LogLinOutputManager&); // no default assignment

protected:
  virtual void Write()=0;
  void MakeBackupDir();
  void CreateDirectory();
  void CheckAndInitializeLogLin();
  Sample *sample;
  unsigned long int blockSize;
  std::string outputDirectory;
  bool createsDirectory;
  std::string baseFilename;
  std::string backup_ext;
  unsigned int maxNumIoBlocks;
  unsigned int numFilenameDigits;
  LogLin logLin;
  bool duplicateBlockEnds;
  bool verbose;
};

#endif // LOGLINOUTPUTMANAGER_H
